import board
import food
import snake
import random
import pygame

FIELD_SIZE = 50
FIELD_DIM = 10

WINDOW_WIDTH = FIELD_SIZE * (FIELD_DIM + 2)
WINDOW_HEIGHT = FIELD_SIZE * (FIELD_DIM + 3)

class Game():
    def __init__(self):
        self.snake = snake.Snake()
        self.board = board.Board(FIELD_DIM, FIELD_DIM, FIELD_SIZE)
        self.food = None
        self.generate_food()

        pygame.init()
        pygame.display.set_caption("Snake game")
        self.window = pygame.display.set_mode([WINDOW_WIDTH, WINDOW_HEIGHT])
        pygame.font.init()
        self.myfont = pygame.font.SysFont('Comic Sans MS', 30)
        
    def get_window(self):
        return self.window

    def try_next_move(self, dx, dy):
        head = self.snake.get_head()
        x = head[0] + dx
        y = head[1] + dy
        
        if(self.snake.possible_move(x,y) and self.board.possible_move(x,y)):
            Game.update(self, x, y)
            return True
        return False

    def generate_food(self):
        newFood = False
        while(not newFood):
            x = random.randint(1, self.board.width)
            y = random.randint(1, self.board.height)
            if (x,y) not in self.snake.body:
                self.food = food.Food(x,y)
                newFood = True

    def check_field(self, dx, dy):
        head = self.snake.get_head()
        x = head[0] + dx
        y = head[1] + dy
        
        if(self.snake.possible_move(x,y) and self.board.possible_move(x,y)):
            return 1
        return 0

    def update(self, x, y):
        if(x == self.food.x and y == self.food.y):
            self.snake.eat(x,y)
            self.generate_food()
        else:
            self.snake.update(x, y)

    def get_distance_to_obstacle(self):
        (x, y) = self.snake.get_head()
        w = self.board.width
        h = self.board.height

        [l, r, u, d] = (x - 1, w - x, y - 1, h - y)

        for (bx, by) in self.snake.body[1:]:
            if x == bx:
                bd = abs(y - by) - 1
                if by > y and bd < d:
                    d = bd
                elif by < y and bd < u:
                    u = bd
            elif y == by:
                bd = abs(bx - x) - 1
                if bx > x and bd < r:
                    r = bd
                elif bx < x and bd < l:
                    l = bd

        return [l, r, u, d]

    def play(self):
        return self.try_next_move(self.snake.orientation[0], self.snake.orientation[1])

    def draw(self):
        self.window.fill((0,0,0))
        self.board.draw(self.window)
        self.snake.draw(self.window, FIELD_SIZE)
        self.food.draw(self.window, FIELD_SIZE)

        text_score = self.myfont.render("Broj nagrada: " + str(self.snake.get_score()), False, (255, 255, 255))
        self.window.blit(text_score, (50, 550))