
import neat
import pygame
import os
import math
import game
import sys

pygame.font.init()
myfont = pygame.font.SysFont('Comic Sans MS', 30)
generation = -1

def simulate_best_genome_result(best_snake_nn):
    g = game.Game()
    window = g.get_window()
    while(g.play()):
        clock = pygame.time.Clock()
        clock.tick(15)
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                pygame.quit()
                sys.exit()

        g.draw()
        text_gen = myfont.render("Generacija: " + str(generation), False, (255, 255, 255))
        window.blit(text_gen, (50, 600))
        pygame.display.update()  

        if(g.snake.steps >= 50):
            break
      
        food_pos = g.food.get_position()
        head = g.snake.get_head()
        x_dif = food_pos[0] - head[0]
        y_dif = food_pos[1] - head[1]
        
        output = best_snake_nn.activate([x_dif, y_dif, *g.get_distance_to_obstacle()])
        g.snake.set_new_orientation(output)

def eval_genoms(genomes, config):
    nns = []
    games = []
    gen = []
    best_snake_nn = None
    best_snake_fitness = 0
    global generation
    generation += 1

    for id, genome in genomes:
        nns.append(neat.nn.FeedForwardNetwork.create(genome, config))
        games.append(game.Game())
        genome.fitness = 0
        gen.append(genome)
    
    while len(games):
        for i, g in enumerate(games):
            if(not best_snake_nn or gen[i].fitness > best_snake_fitness):
                best_snake_nn = nns[i]
            
            if(g.snake.steps >= 50):
                games.pop(i)
                nns.pop(i)
                gen.pop(i)
                continue
            
            food_pos = g.food.get_position()
            head = g.snake.get_head()
            x_dif = food_pos[0] - head[0]
            y_dif = food_pos[1] - head[1]

            nn = nns[i]
            output = nn.activate([x_dif, y_dif, *g.get_distance_to_obstacle()])
            g.snake.set_new_orientation(output)
            
            if (g.play() == False):
                games.pop(i)
                nns.pop(i)
                gen.pop(i)
                continue
            
            head = g.snake.get_head()
            x_dif_new = food_pos[0] - head[0]
            y_dif_new = food_pos[1] - head[1]

            if(x_dif_new == 0 and y_dif_new == 0):
                gen[i].fitness += 1

            distance_before = math.sqrt(math.pow(x_dif, 2) + math.pow(y_dif, 2))
            distance_after = math.sqrt(math.pow(x_dif_new, 2) + math.pow(y_dif_new, 2))
            
            if(distance_after < distance_before):
                gen[i].fitness += 0.02
            else:
                gen[i].fitness -= 0.01
            
    simulate_best_genome_result(best_snake_nn)

def run(config_file):
    config = neat.Config(neat.DefaultGenome, neat.DefaultReproduction, neat.DefaultSpeciesSet, neat.DefaultStagnation, config_file)
    p = neat.Population(config)
    p.add_reporter(neat.StdOutReporter(True))
    stats = neat.StatisticsReporter()
    p.add_reporter(stats)
    winner = p.run(eval_genoms, 100)
    print(stats.get_fitness_mean())
    stats.save_genome_fitness(',')
    stats.save_species_count(',')
    stats.save_species_fitness(',')
    with open('genom.txt', 'w') as file_gen:
        file_gen.write('{!s}'.format(winner))
        file_gen.write('')
        file_gen.write('{!s}'.format(stats.best_genome()))
    print('\nBest genome:\n{!s}'.format(winner))
    print('\nBest genome:\n{!s}'.format(stats.best_genome()))

if __name__ == "__main__":
    local_dir = os.path.dirname(__file__)
    config_path = os.path.join(local_dir, 'config-snake.txt')
    run(config_path)
